@@ -2,7 +2,7 @@ local noder     = require 'core.noder'
 local guide     = require 'parser.guide'
 local files     = require 'files'
 local generic   = require 'core.generic'
-local ws        = require 'workspace'
+local rpath     = require 'workspace.require-path'
 local vm        = require 'vm.vm'
 local collector = require 'core.collector'
 local util      = require 'utility'
@@ -17,6 +17,7 @@ local next         = next
 local error        = error
 local type         = type
 local setmetatable = setmetatable
+local ipairs       = ipairs
 local tconcat      = table.concat
 local ssub         = string.sub
 local sfind        = string.find
@@ -39,6 +40,7 @@ local getHeadID         = noder.getHeadID
 local eachForward       = noder.eachForward
 local getUriAndID       = noder.getUriAndID
 local eachBackward      = noder.eachBackward
+local eachExtend        = noder.eachExtend
 local eachSource        = noder.eachSource
 local compileAllNodes   = noder.compileAllNodes
 local compilePartNoders = noder.compilePartNodes
@@ -191,17 +193,17 @@ local pushRefResultsMap = util.switch()
 ---@param force  boolean
 local function pushResult(status, mode, source, force)
     if not source then
-        return
+        return false
     end
     local results = status.results
     local mark = status.rmark
     if mark[source] then
-        return
+        return true
     end
     mark[source] = true
     if force then
         results[#results+1] = source
-        return
+        return true
     end
 
     if mode == 'def'
@@ -209,7 +211,7 @@ local function pushResult(status, mode, source, force)
         local f = pushDefResultsMap[source.type]
         if f and f(source, status) then
             results[#results+1] = source
-            return
+            return true
         end
     elseif mode == 'ref'
     or     mode == 'field'
@@ -218,7 +220,7 @@ local function pushResult(status, mode, source, force)
         local f = pushRefResultsMap[source.type]
         if f and f(source, status) then
             results[#results+1] = source
-            return
+            return true
         end
     end
 
@@ -226,9 +228,11 @@ local function pushResult(status, mode, source, force)
     if parent.type == 'return' then
         if source ~= status.source then
             results[#results+1] = source
-            return
+            return true
         end
     end
+
+    return false
 end
 
 ---@param obj parser.guide.object
@@ -736,6 +740,46 @@ function m.searchRefsByID(status, suri, expect, mode)
         end
     end
 
+    local function checkExtend(uri, id, field)
+        if  not field
+        and mode ~= 'field'
+        and mode ~= 'allfield' then
+            return
+        end
+        if field then
+            local results = status.results
+            for i = #results, 1, -1 do
+                local res = results[i]
+                if res.type == 'setfield'
+                or res.type == 'setmethod'
+                or res.type == 'setindex' then
+                    local resField = noder.getFieldID(getID(res))
+                    if field == resField then
+                        return
+                    end
+                end
+                if res.type == 'doc.field.name' then
+                    local resField = STRING_FIELD .. res[1]
+                    if field == resField then
+                        return
+                    end
+                end
+            end
+        end
+        for extendID in eachExtend(nodersMap[uri], id) do
+            local targetUri, targetID
+
+            targetUri, targetID = getUriAndID(extendID)
+            if targetUri and targetUri ~= uri then
+                if dontCross == 0 then
+                    searchID(targetUri, targetID, field, uri)
+                end
+            else
+                searchID(uri, targetID or extendID, field)
+            end
+        end
+    end
+
     local function searchSpecial(uri, id, field)
         -- Special rule: ('').XX -> stringlib.XX
         if id == 'str:'
@@ -752,7 +796,7 @@ function m.searchRefsByID(status, suri, expect, mode)
         if not requireName then
             return
         end
-        local uris = ws.findUrisByRequirePath(requireName)
+        local uris = rpath.findUrisByRequirePath(requireName)
         footprint(status, 'require:', requireName)
         for i = 1, #uris do
             local ruri = uris[i]
@@ -774,7 +818,8 @@ function m.searchRefsByID(status, suri, expect, mode)
         local crossed = {}
         if mode == 'def'
         or mode == 'alldef'
-        or field then
+        or field
+        or hasCall(field) then
             for _, guri in ceach('def:' .. id) do
                 if uri == guri then
                     goto CONTINUE
@@ -782,6 +827,22 @@ function m.searchRefsByID(status, suri, expect, mode)
                 searchID(guri, id, field, uri)
                 ::CONTINUE::
             end
+        elseif mode == 'field'
+        or     mode == 'allfield' then
+            for _, guri in ceach('def:' .. id) do
+                if uri == guri then
+                    goto CONTINUE
+                end
+                searchID(guri, id, field, uri)
+                ::CONTINUE::
+            end
+            for _, guri in ceach('field:' .. id) do
+                if uri == guri then
+                    goto CONTINUE
+                end
+                searchID(guri, id, field, uri)
+                ::CONTINUE::
+            end
         else
             for _, guri in ceach(id) do
                 if crossed[guri] then
@@ -850,7 +911,6 @@ function m.searchRefsByID(status, suri, expect, mode)
     local function searchNode(uri, id, field)
         local noders = nodersMap[uri]
         local call   = noders.call[id]
-        local global = isGlobalID(id)
         callStack[#callStack+1] = call
 
         if field == nil and not ignoredSources[id] then
@@ -865,8 +925,8 @@ function m.searchRefsByID(status, suri, expect, mode)
             checkRequire(uri, requireName, field)
         end
 
-        local elock = global and elockMap['@global'] or elockMap[uri]
-        local ecall = global and ecallMap['@global'] or ecallMap[uri]
+        local elock = elockMap[uri]
+        local ecall = ecallMap[uri]
 
         if lockExpanding(elock, ecall, id, field) then
             if noders.forward[id] then
@@ -875,6 +935,9 @@ function m.searchRefsByID(status, suri, expect, mode)
             if noders.backward[id] then
                 checkBackward(uri, id, field)
             end
+            if noders.extend[id] then
+                checkExtend(uri, id, field)
+            end
             releaseExpanding(elock, ecall, id, field)
         end
 
@@ -960,6 +1023,7 @@ function m.searchRefsByID(status, suri, expect, mode)
     end
 
     search(suri, expect, nil)
+    flushGeneric()
 end
 
 local function prepareSearch(source)
@@ -975,6 +1039,22 @@ local function prepareSearch(source)
     end
     local uri  = getUri(source)
     local id   = getID(source)
+    -- return function
+    if source.type == 'function' and source.parent.type == 'return' then
+        local func = guide.getParentFunction(source)
+        if func.type == 'function' then
+            for index, rtn in ipairs(source.parent) do
+                if rtn == source then
+                    id = sformat('%s%s%s'
+                        , getID(func)
+                        , RETURN_INDEX
+                        , index
+                    )
+                    break
+                end
+            end
+        end
+    end
     return uri, id
 end
 
