@@ -763,7 +763,7 @@ end
 
 function m.positionToOffsetByLines(lines, position)
     local row, col = m.rowColOf(position)
-    return lines[row] + col - 1
+    return (lines[row] or 1) + col - 1
 end
 
 --- 返回全文光标位置
@@ -781,7 +781,7 @@ function m.offsetToPositionByLines(lines, offset)
         row = (left + right) // 2
         if row == left then
             if right ~= left then
-                if lines[right] <= offset then
+                if lines[right] - 1 <= offset then
                     row = right
                 end
             end
@@ -1151,4 +1151,26 @@ function m.isGlobal(source)
     return false
 end
 
+function m.isInString(ast, position)
+    return m.eachSourceContain(ast, position, function (source)
+        if  source.type == 'string'
+        and source.start < position then
+            return true
+        end
+    end)
+end
+
+function m.isOOP(source)
+    if source.type == 'setmethod'
+    or source.type == 'getmethod' then
+        return true
+    end
+    if source.type == 'method'
+    or source.type == 'field'
+    or source.type == 'function' then
+        return m.isOOP(source.parent)
+    end
+    return false
+end
+
 return m
